//
//  FPViewer.h
//  ViewerControl
//
//  Created by Jan-Peter Vos on 08/12/15.
//  Copyright © 2015 Floorplanner.com. All rights reserved.
//

#import <UIKit/UIKit.h>
#import <GLKit/GLKit.h>

@class FPViewer;

typedef NS_ENUM(NSInteger, FPViewerViewMode) {
    FPViewerViewMode2D,
    FPViewerViewMode3D
};

typedef NS_ENUM(NSInteger, FPCameraMode) {
    FPCameraModeOverview,
    FPCameraModeWalkthrough
};

typedef NS_ENUM(NSInteger, FPRenderStyle) {
    FPRenderStyleTextured,
    FPRenderStyleSketch
};

typedef NS_ENUM(NSInteger, FPViewerErrorType) {
    FPViewerErrorTypeProjectLoading,
    FPViewerErrorTypeGetFloors,
    FPViewerErrorTypeCreateScene,
    FPViewerErrorTypeDownloadProject
};

@interface FPDesign : NSObject

@property int id;
@property (nonnull) NSString* name;

@end


@interface FPFloor : NSObject

@property int id;
@property (nonnull) NSString* name;

@property (nonnull) NSArray<FPDesign*>* designs;

@end


@protocol FPViewerDelegate<NSObject>

@optional

- (void)viewer:(FPViewer* _Nonnull)viewer hasEncounteredError:(NSError* _Nullable)error ofType:(FPViewerErrorType)type;
- (void)viewerDidFinishLoadProject:(FPViewer* _Nonnull)viewer;
- (void)viewer:(FPViewer* _Nonnull)viewer didShowDesignWithViewMode:(FPViewerViewMode)viewMode;

@end


@interface FPViewer : UIView

@property (getter=getViewMode, setter=setViewMode:) FPViewerViewMode viewMode;
@property (getter=getCameraMode, setter=setCameraMode:) FPCameraMode cameraMode;
@property (getter=getRenderStyle, setter=setRenderStyle:) FPRenderStyle renderStyle;
@property (getter=isRenderStyleSketchSupported, readonly) BOOL renderStyleSketchSupported;
@property (nonatomic, weak, nullable) id <FPViewerDelegate> delegate;

@property (getter=getWallsSection, setter=setWallsSection:) IBInspectable NSInteger wallsSection;
@property (getter=xRayWallsEnabled, setter=setXRayWallsEnabled:) IBInspectable BOOL xRayWallsEnabled;
@property (getter=showGrid, setter=setShowGrid:) IBInspectable BOOL showGrid;
@property (getter=showLabels, setter=setShowLabels:) IBInspectable BOOL showLabels;
@property (getter=showObjects, setter=setShowObjects:) IBInspectable BOOL showObjects;
@property (getter=engineAutoDims, setter=setEngineAutoDims:) IBInspectable BOOL engineAutoDims;
@property (getter=showDims, setter=setShowDims:) IBInspectable BOOL showDims;

- (NSArray<FPFloor*>* _Nullable) getFloors;
- (void)getFloors:(nonnull void(^)(NSArray<FPFloor*>* _Nullable, NSError* _Nullable))callback;
- (void)loadProjectWithId:(NSInteger)designId;
- (void)loadProjectFromUrl:(nonnull NSURL*)url;
- (void)loadDesign:(nonnull FPDesign*)design;

- (void)setBackgroundGradient3D:(NSArray<UIColor*>* _Nonnull)colors;

@end
